/*************************************
	Sandstorm
	BY:  OMAR METWALLY
	START DATE:  May 19, '01
	COMPLETED:   August 19, '01

	MAIN.C
**************************************/


/**** INCLUDES ****/
#include <libps.h>
#include "main.h"
#include "graphics.h"
#include "pad.h"
#include "control.h"
#include "debug.h"
#include "sprite.h"
#include "collide.h"  
#include "fire.h"
#include "sincos.h"
#include "ai.h"
#include "cdread.h"
#include "sound.h"

/**** GLOBALS ****/

//graphics related
int out_buf;
GsOT wot[2];				//OT headers
GsOT_TAG ot[2][1<<OTLENGTH];		//actual OTs
PACKET gpu_work_area[2][PACKETMAX];	//GPU Packet work area
u_char mode;				//check for video mode

//camera and light related
GsRVIEW2 view;
GsF_LIGHT light[3];
GsFOGPARAM fog;

//sprite instances
GsSPRITE tree[5], leftbg, rightbg, sun2, upg_ico, mine_ico, fire_ico, hlt_ico,
	hom_ico, mine_ico, pow_ico, rico_ico, tbo_ico, time_ico, bullet[21], mine_1, mine_2,
	mine_3, ricochet, os_hom, os_fire, os_pow, os_rico, os_mine, os_bullet, os_time, fire1, fire2, 
	fire3a, fire3b, fire4a, fire4b, fire5a, fire5b;
GsBOXF le1, le2, le3, life, turbo, cpulife1, cpulife2, radar, blip, blip2, dot, blank;

//object structures
struct object sand, track, truck[NUMCARS], pyramid, pointer, homing_m, power_m, fire_m,
	homing2_m, power2_m, fire2_m, homing3_m, power3_m, fire3_m;
struct DB buff[2];
struct inventory p1items;		//inventory management
struct sphere redcar, greencar, whitecar, item, missile, bulletsphere, mine_expl, car_coll;

GsCOORDINATE2 oldcoord0, oldcoord1, oldcoord2;	//restoring car positions in colllision

//game related
int gameover, pause; 
volatile u_char *bb0,*bb1;
u_long padd,padd2;
int gamemode, level, wincount;	//level is difficulty level; 0-easy; 1-normal; 2-hard
int debugmode;
short bull_stat[21];
int timelimit[9];	//used for item managment 
int rot=(90<<12);	//car rotation variable
int spot[9];		//locations of items
int bull_dir[21];	//directions for projectiles
short sel_item, sel_item_old, lock_sel9, lock;		//selected item
int x_speed, z_speed, rico_life;	//ricochet variables
int rico_status, rico_dir;
int mine_status, mine_dir;
int picksprite;
int fire_m_status, fired2_m_status, homing_m_status, power_m_status, fire_m_dir, homing_m_dir, power_m_dir,
	fire2_m_status, homing2_m_status, power2_m_status, fire2_m_dir, homing2_m_dir, power2_m_dir,
	fire3_m_status, homing3_m_status, power3_m_status, fire3_m_dir, homing3_m_dir, power3_m_dir, firestatus;

int freeze, actcounter1, actcounter2, actcounter3, actcounter4, actcounter5, actcounter6,
	expl_status, expl_status2, expl_status3, detonate;	
int p1destroyed, cpu1destroyed, cpu2destroyed;	

//item assignments: 0-MG upgrade; 1-mine; 2- fire missile; 3-health; 4-homing; 5-power;
//		    6-ricochet; 7-turbo; 8-time; 9-NULL

//2d object pos in world
VECTOR pos[5], sunpos, upg_ico_pos, mine_ico_pos, fire_ico_pos, hlt_ico_pos,
	hom_ico_pos, pow_ico_pos, rico_ico_pos, tbo_ico_pos, time_ico_pos, bullet_pos[21],
	mine_1_pos, mine_2_pos, mine_3_pos, rico_pos, fire1_pos, fire2_pos;

extern int state1, state2;
extern int vis_3, vis_4, vis_5;
extern int vis_3_count, vis_4_count, vis_5_count;

//AI attack state control
int fired1_p, fired1_f, fired1_h, homing2_status, homing2_count, fire2_h_count, fire2_h_status;
int fired2_p, fired2_f, fired2_h, homing3_status, homing3_count, fire3_h_count, fire3_h_status;

int limit, limit2;	//AI smoother


/**** MAIN FUNCTION ****/

int main( void )
{
	int x;
	
	InitGraphics();
	InitCoordinates();
	
	//create the sand quadrillateral for egypt track
	CreateTMD(&sand.obj,&sand.coord,(u_long*)SAND_DATA);
	CreateTMD(&truck[0].obj,&truck[0].coord,(u_long*)TRUCKG_DATA);
	CreateTMD(&truck[1].obj,&truck[1].coord,(u_long*)TRUCKR_DATA);
	CreateTMD(&truck[2].obj,&truck[2].coord,(u_long*)TRUCKW_DATA);
	CreateTMD(&pyramid.obj,&pyramid.coord,(u_long*)PYRAMID_DATA);
	CreateTMD(&pointer.obj, &pointer.coord, (u_long*)POINTER_DATA);
	CreateTMD(&homing_m.obj, &homing_m.coord, (u_long*)HOMING_M_DATA);
	CreateTMD(&fire_m.obj, &fire_m.coord, (u_long*)FIRE_M_DATA);
	CreateTMD(&power_m.obj, &power_m.coord, (u_long*)POWER_M_DATA);
	CreateTMD(&homing2_m.obj, &homing2_m.coord, (u_long*)HOMING_M_DATA);
	CreateTMD(&fire2_m.obj, &fire2_m.coord, (u_long*)FIRE_M_DATA);
	CreateTMD(&power2_m.obj, &power2_m.coord, (u_long*)POWER_M_DATA);
	CreateTMD(&homing3_m.obj, &homing3_m.coord, (u_long*)HOMING_M_DATA);
	CreateTMD(&fire3_m.obj, &fire3_m.coord, (u_long*)FIRE_M_DATA);
	CreateTMD(&power3_m.obj, &power3_m.coord, (u_long*)POWER_M_DATA);

	InitTex((u_long*)SANDTEX_DATA);

	InitView();
	InitLights();
	
	for(x=0;x<5;x++)
		InitObject2D( (u_long*)BIRTCH_DATA, &tree[x], (1<<24), 0, 0, 98, 131, 0, 0 );

	//initialise 2D sprite objects
	InitObject2D( (u_long*)LEFTSKY_DATA, &leftbg, (1<<24), -160, -120, 160, 240, 0, 0 );
	InitObject2D( (u_long*)RIGHTSKY_DATA, &rightbg, (1<<24), 0, -120, 160, 240, 0, 0 );
	InitObject2D( (u_long*)SUN2_DATA, &sun2, (1<<24), 0, 0, 52, 54, 0, 0 );
	InitObject2D( (u_long*)MEGA_DATA, &upg_ico, (1<<24), 0, 0, 38, 38, 0, 76 );
	InitObject2D( (u_long*)MEGA_DATA, &mine_ico, (1<<24), 0, 0, 38, 38, 114, 0 );
	InitObject2D( (u_long*)MEGA_DATA, &fire_ico, (1<<24), 0, 0, 38, 38, 0, 0 );
	InitObject2D( (u_long*)MEGA_DATA, &hlt_ico, (1<<24), 0, 0, 38, 38, 38, 0 );
	InitObject2D( (u_long*)MEGA_DATA, &hom_ico, (1<<24), 0, 0, 38, 38, 76, 0 );
	InitObject2D( (u_long*)MEGA_DATA, &pow_ico, (1<<24), 0, 0, 38, 38, 0, 38 );
	InitObject2D( (u_long*)MEGA_DATA, &rico_ico, (1<<24), 0, 0, 38, 38, 38, 38 );
	InitObject2D( (u_long*)MEGA_DATA, &tbo_ico, (1<<24), 0, 0, 38, 38, 76, 38 );
	InitObject2D( (u_long*)MEGA_DATA, &time_ico, (1<<24), 0, 0, 38, 38, 114, 38 );
	for(x=0; x<21; x++)
		InitObject2D( (u_long*)MEGA_DATA, &bullet[x], (1<<24), 0, 0, 8, 7, 38, 76 );
	InitObject2D( (u_long*)MEGA_DATA, &mine_1, (1<<24), 0, 0, 18, 39, 46, 75 );
	InitObject2D( (u_long*)MEGA_DATA, &mine_2, (1<<24), 0, 0, 18, 39, 64, 76 );
	InitObject2D( (u_long*)MEGA_DATA, &mine_3, (1<<24), 0, 0, 18, 39, 82, 76 );
	InitObject2D( (u_long*)MEGA_DATA, &ricochet, (1<<24), 0, 0, 38, 38, 100, 76 );
	
	//initialise onscreen icons
	InitObject2D( (u_long*)ONSCREEN_DATA, &os_pow, (1<<24), 0, 0, 25, 38, 25, 0 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &os_fire, (1<<24), 0, 0, 25, 38, 0, 0 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &os_hom, (1<<24), 0, 0, 25, 38, 50, 0 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &os_mine, (1<<24), 0, 0, 38, 38, 75, 0 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &os_bullet, (1<<24), 0, 0, 38, 38, 0, 38 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &os_rico, (1<<24), 0, 0, 38, 38, 38, 38 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &os_time, (1<<24), 0, 0, 38, 38, 76, 38 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire1, (1<<24), 0, 0, 32, 32, 0, 82 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire2, (1<<24), 0, 0, 32, 32, 32, 82 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire3a, (1<<24), 0, 0, 32, 32, 0, 82 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire3b, (1<<24), 0, 0, 32, 32, 32, 82 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire4a, (1<<24), 0, 0, 32, 32, 0, 82 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire4b, (1<<24), 0, 0, 32, 32, 32, 82 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire5a, (1<<24), 0, 0, 32, 32, 0, 82 );
	InitObject2D( (u_long*)ONSCREEN_DATA, &fire5b, (1<<24), 0, 0, 32, 32, 32, 82 );

	//initialise GsBOXFs for energy bars and lens flare elements
	InitBox( &le1, (1<<30), -35, -35, 30, 30*240/320,  255, 255, 180 );
	InitBox( &le2, (1<<30), 0, 0, 20, 20*240/320, 255, 255, 180 );
	InitBox( &le3, (1<<30), 25, 25, 10, 10*240/320, 255, 255, 180 );
	InitBox( &turbo, (1<<30), 0, 0, 11, 0, 8, 8, 240 );
	InitBox( &life, (1<<30), -147, 66, 80, 16, 240, 0, 0 );
	//InitBox( &radar, (1<<30), -147, -109, 84, 58, 192, 192, 192 );
	//InitBox( &blip, (1<<30), -147, -109, 3, 3, 240, 0, 0 );
	//InitBox( &blip2, (1<<30), -147, -111, 3, 3, 240, 0, 0 );
	InitBox( &cpulife1, (0<<30), 0, 0, 40, 8, 0, 240, 0 );
	InitBox( &cpulife2, (0<<30), 0, 0, 40, 8, 0, 240, 0 );
	InitBox( &dot, (1<<30), -102, -18, 7, 5, 240, 0, 0 );
	InitBox( &blank, (2<<28), -200, -200, 400, 400, 0, 0, 0 );
	
	//read VB and VH files from CD
	//DataFileSearch();
	//DataFileRead();
	//printf("\nCD-ROM read complete\n");

	//init BG music
	InitSound();

	//initialises object positions and attributes--last function call before loop
	InitPos();
	

	// main while loop
	while(gameover==0)
	{
		padd=(~(*(bb0+3) | *(bb0+2) << 8));	//read pad 1
		padd2=(~(*(bb1+3) | *(bb1+2) << 8));	//read pad 2
		
		oldcoord0=truck[0].coord;
		oldcoord1=truck[1].coord;
		oldcoord2=truck[2].coord;

		switch(gamemode) {
			case 1: 	//title screen mode
			{
				DisplayText();
				MenuSystem();
				break;
			}
			case 2:		//one player mode
			{

		if(pause==0) {		//pause=1 stops all cars from moving
		MoveCar();
		SetState(&state1);
		SetState(&state2);
		
		if(cpu1destroyed==1 || freeze==1) state1=10;		//set state value to neutral value
		if(cpu2destroyed==1 || freeze==1) state2=10;	
		
		CPU1Reaction(&state1,&state2);
	
		if(cpu1destroyed==1) truck[1].coord.coord.t[2]=3500;
		if(cpu2destroyed==1) truck[2].coord.coord.t[2]=3500;

		//if difficulty level is "hard"
		if(level==2) {
		CPUAttack( &truck[1].coord, &truck[0].coord, &fired1_p, &fired1_f, &fired1_h, &state1, 1, 4, 5, 7 );
		CPUAttack( &truck[2].coord, &truck[0].coord, &fired2_p, &fired2_f, &fired2_h, &state2, 2, 3, 6, 8 );
		}

		//if difficulty level is "normal"
		if(level==1) {
		CPUAttack( &truck[1].coord, &truck[0].coord, &fired1_p, &fired1_f, &fired1_h, &state1, 10, 4, 5, 7 );
		CPUAttack( &truck[2].coord, &truck[0].coord, &fired2_p, &fired2_f, &fired2_h, &state2, 10, 3, 6, 8 );
		}

		//if difficulty level is "easy"
		if(level==0) {
		CPUAttack( &truck[1].coord, &truck[0].coord, &fired1_p, &fired1_f, &fired1_h, &state1, 10, 4, 10, 7 );
		CPUAttack( &truck[2].coord, &truck[0].coord, &fired2_p, &fired2_f, &fired2_h, &state2, 10, 3, 10, 8 );
		}

		PosFireCars();

		//stop from moving or firing if "frozen"
		if(freeze==1) { 
			truck[1].coord=oldcoord1; truck[2].coord=oldcoord2; 
			fired1_p=fired1_f=fired1_h=0;
			fired2_p=fired2_f=fired2_h=0;	
		}	

		FireCPUMissile();
		FireCPUMissile2();

		ManageItems();
		PickUpItem();
		ChangeItem();

		FireBullet();
		MoveBullet();
		Ricochet();
		PlaceMine();
		FireMissile();

		RotMissile( &fire_m.vector, &fire_m_status );
		RotMissile( &homing_m.vector, &homing_m_status);
		RotMissile( &power_m.vector, &power_m_status);

		RotMissile( &fire2_m.vector, &fire2_m_status );
		RotMissile( &homing2_m.vector, &homing2_m_status );
		RotMissile( &power2_m.vector, &power2_m_status );

		RotMissile( &fire3_m.vector, &fire3_m_status );
		RotMissile( &homing3_m.vector, &homing3_m_status );
		RotMissile( &power3_m.vector, &power3_m_status );		
		
		TestAllMissiles();
		
		} //end if(pause==0)

		HideFire();
		//DisplayRadar( &blip, &blip2 );
		AssessDamage();

		
		//smooth item change
		if(lock_sel9==1) {
			sel_item=9;
			lock++;
			if(lock>12) { lock=0; sel_item=0; lock_sel9=0; }
		}

		//restore coordinates after collision before drawing screen
		LimitArea( &truck[0].coord, &oldcoord0 );
		LimitArea( &truck[1].coord, &oldcoord1 );
		LimitArea( &truck[2].coord, &oldcoord2 );
		TestCarCollision();
	
		//draw fog
		//GsSetFogParam(&fog);

		UpdateCoordinates(&sand.vector,&sand.coord);
		UpdateCoordinates(&track.vector,&track.coord);
		for(x=0;x<NUMCARS;x++)
			UpdateCoordinates(&truck[x].vector,&truck[x].coord);
		UpdateCoordinates(&pyramid.vector,&pyramid.coord);
		UpdateCoordinates(&pointer.vector, &pointer.coord);
		UpdateCoordinates(&power_m.vector, &power_m.coord);
		UpdateCoordinates(&fire_m.vector, &fire_m.coord);
		UpdateCoordinates(&homing_m.vector, &homing_m.coord);

		UpdateCoordinates(&power2_m.vector, &power2_m.coord);
		UpdateCoordinates(&fire2_m.vector, &fire2_m.coord);
		UpdateCoordinates(&homing2_m.vector, &homing2_m.coord);
	
		UpdateCoordinates(&power3_m.vector, &power3_m.coord);
		UpdateCoordinates(&fire3_m.vector, &fire3_m.coord);
		UpdateCoordinates(&homing3_m.vector, &homing3_m.coord);
		
		} //end case 2:
	
		} //end entire switch-case
		
		Fade();
		DrawScreen();
		SwitchMode();
		MoveCam();

	} //end while loop
	
	DeInit();
	return(0);

}// end main 


	
